function [z, v, K_plus] = CRAFT(X, lambda, d_num, m, rho, MaxIter, seed, ApproxK)

%This code implements the cluster-specific feature selection algorithm described in the following paper:
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%V. K. Garg, C. Rudin, and T. Jaakkola. CRAFT: ClusteR-specific Assorted Feature selecTion, AISTATS, 2016.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Inputs:
%
%X:         (n x d) input data matrix, where row i denotes a d-dimensional instance x_i. 
%lambda:    regularization parameter (should be positive, seeApproxK too).
%d_num:     total number of numeric features. We assume that the features 1 to
%           d_num are numeric, and features d_num+1 onwards are categorical binary. 0 <= d_num <= d 
%MaxIter:   maximum number of iterations allowed.
%m:         average fraction of features per cluster (should be in the range (0, 1)). 
%rho:       (optional) global/local feature control parameter (should be in the range (0, m(1-m)).      
%seed:      (optional)initial configuration for pseudorandom number
%           generator - included for reasons of reproducibility. 
%ApproxK:   (optional) Approximate number of clusters. CRAFT is nonparametric and so this parameter is not required. However, in order to compare with other algorithms, 
%           it often helps to have a parameterized version. If ApproxK > 1, we ignore the input lambda, and instead compute an approximate lambda using the farthest-first heuristic 
%           decribed in the paper.
%
%Outputs:
%
%K_plus:    Number of output clusters 
%z:         Cluster assignment vector
%v:         (K_plus x d) Boolean matrix; v(k, j) = 1 if feature number j is selected for cluster k.  
%
%If you find this work useful, please consider citing the following paper:
%
%V. K. Garg, C. Rudin, and T. Jaakkola. CRAFT: ClusteR-specific Assorted Feature selecTion, AISTATS, 2016.
%
%Code implemented by: Vikas Garg (http://www.mit.edu/~vgarg/)    



% Default parameter settings
if nargin < 8, ApproxK = 0; end
if nargin < 7, seed = 1; end
if nargin < 6, MaxIter = 100; end 
if nargin < 5, rho = m*(1-m)-0.01; end 

rho = min(rho, m*(1-m)-0.01);                       % adjust rho if it is not in the valid range.
rng(seed);
row = size(X, 1);                                   % total number of data points (instances)
d = size(X, 2);                                     % total number of features (both numeric and categorical included).
d_cat = d - d_num;                                  % number of categorical features
mean_global = mean(X, 1);
sample = randperm(row, 1); 
s = rng;                                            

% Define variables used in eq (1) of the paper
a0 = m^2 * (1-m)/rho - m;
b0 = m * (1-m)^2/rho + m;
a1 = a0+1;
b1 = b0-1;
F0 = (a0+b0).* log(a0+b0+eps) - a0.*log(a0+eps) - b0.*log(b0+eps);
F1 = (a1+b1).* log(a1+b1+eps) - a1.*log(a1+eps) - b1.*log(b1+eps);
FDelta = F1 - F0;


% Compute an approximate lambda corresponding to ApproxK
if ApproxK > 1
    lambda = FindApproximateLambda(X, m, ApproxK, d_num, FDelta, a0, b0, seed);
end


z = ones(row, 1);                                  % initialize all points to have the same cluster label.
z_prev = z;                                        % to keep track of cluster assignments in the previous iteration 
mu = X(sample, :);                                 % first center is taken to be a random sample 
mu_num = mu(:, 1:d_num);                           
mu_cat = mu(:, d_num+1:d);                         
variance = ones(1, d_num);                          
X_num = X(:, 1:d_num);                             % extract numeric features
X_cat = X(:, d_num+1:d);                           % extract categorical features 
rng(s);                                             
v = binornd(1, m*ones(1, d));
v_num = v(:, 1:d_num);
v_cat = v(:, d_num+1:d);
mean_global_cat = mean_global(:, d_num+1:end);
k = 1;                                             % initially there is a single cluster

for iter = 1:1:MaxIter
    for n = 1:1:row
        global_contribution = -X_cat(n, :).*log(mean_global_cat+eps) - (1-X_cat(n, :)).*log(1-mean_global_cat+eps);
        local_contribution = - X_cat(n, :).*log(mu_cat(1, :) + eps) - (1-X_cat(n, :)).*log(1- mu_cat(1, :)+eps);
        total_num = sum(v_num(1, :).*((mu_num(1, :) - X_num(n, :)) .^2)./(2*variance(1, :)));
        min_dist = sum(v(1, :), 2)*FDelta + sum(v_cat(1, :) .* local_contribution) + sum((1-v_cat(1, :)).* global_contribution) + total_num;      
        min_index = 1;
        for c = 2:1:k                               % find the closest cluster
            local = - X_cat(n, :).*log(mu_cat(c, :) + eps) - (1-X_cat(n, :)).*log(1- mu_cat(c, :) +eps);
            total_num_c = sum(v_num(c, :).*((mu_num(c, :) - X_num(n, :)) .^2)./(2*variance(c, :)));
            dist = sum(v(c, :), 2)*FDelta + sum(v_cat(c, :) .* local) + sum((1-v_cat(c, :)).* global_contribution) + total_num_c;
            if dist < min_dist
               min_index = c;                  
               min_dist = dist;
            end
        end
        if min_dist > lambda + d*F0
           k = k + 1;                               % start a new cluster
           z(n) = k;
           mu = [mu; X(n, :)];
           mu_num = mu(:, 1:d_num);
           variance = [variance; ones(1, d_num)];
           mu_cat = mu(:, d_num+1:d);
           prob = ((k-1)*a0 + sum(v, 1))./((k-1+eps)*(a0+b0+eps)); 
           t_est = binornd(1, prob); 
           v = [v; t_est];
           v_num = v(:, 1:d_num);
           v_cat = v(:, d_num+1:d);
        else
           z(n) = min_index;                         % assign to the closest existing cluster
        end
    end  
    if z_prev == z
       break;                                       % cluster assignments did not change in successive iterations, so done!
    end
    mu = [];
    v = [];
    count = 0;
    valid_index = zeros(1, k);
    for i=1:1:k
        l = X(z==i, :);
        if size(l, 1) > 0                           % make sure there are no empty clusters
           valid_index(1, i) = 1;
           count = count + 1;                       % assign labels starting from 1
           z(z==i) = count;
           mu_l = sum(l, 1) /size(l, 1);            % compute mean of points in  cluster l
           mu = [mu; mu_l];
           mu_num = mu(:, 1:d_num);
           mu_cat = mu(:, d_num+1:d);
           mean_local_mat = repmat(mu_l(:, d_num+1:d), size(l, 1), 1);
           mean_repeat = repmat(mean_global_cat, size(l, 1), 1);
           l_cat = l(:, d_num+1:d);
           g_term = -l_cat.*log(mean_repeat+eps) - (1-l_cat).*log(1-mean_repeat+eps);
           global_vector = sum(g_term, 1);
           l_term = -l_cat.*log(mean_local_mat+eps) - (1-l_cat).*log(1-mean_local_mat+eps);
           local_vector =  sum(l_term, 1);
           difference = global_vector-local_vector;
           [~, Ind_cat] = sort(difference, 2, 'descend');     %determine categorical features for the cluster      
           variance_l = std(l(:, 1:d_num), 0, 1).^2;
           variance = [variance; variance_l];
           [~, Ind_num] = sort(variance_l, 2, 'ascend');      %determine numeric features for the cluster
           Part_cat = Ind_cat(:, 1:m*d_cat); 
           Part_num = Ind_num(:, 1:m*d_num);
           Part_Ind = [Part_num, Part_cat + d_num];   % we need to add d_num to indices obtained from part_cat since Ind_cat indexing starts from 1.
           temp_t = zeros(1, d);
           temp_t(Part_Ind) = 1;
           v = [v; temp_t];
           v_num = v(:, 1:d_num);
           v_cat = v(:, d_num+1:d);
        end
    end
    k = count;
    z_prev = z;
end

K_plus = numel(unique(z));


